;; Bouncing balls

; Keep track of the position (x,y) and direction (xd,yd) of the ball
; Reduce yd by the amount bounce-gravity for each tick

(define bounce-width 600)
(define bounce-height 400)
(define bounce-duration 5000)
(define bounce-size 50)
(define bounce-gravity 50)

; (x,y) is starting position
; (xd,yd) is direction
; yg is gravity
; yv is initial velocity after bounce
; t is starting time
; td is time between ticks
(define (draw-bounce x y xd yd yv t td)
  (ani-properties ANI_VISIBLE 0)
  (ani-time t)
  (ani-properties ANI_VISIBLE 1)
  (ani-properties ANI_WIDTH bounce-size ANI_HEIGHT bounce-size)
  (while (and (<= x bounce-width) (>= x 0) (<= t bounce-duration))
    (ani-time t)
    (ani-properties ANI_X x ANI_Y (- bounce-height y bounce-size))
    (set! x (+ x xd))
    (set! y (+ y yd))
    (if (<= y 0)
      (begin
	(set! yd yv)
	(set! y 0))
      (set! yd (- yd bounce-gravity)))
    (set! t (+ t td)))
  (ani-properties ANI_VISIBLE 0))

(define (draw-sin x y xd yd t td)
  (ani-properties ANI_VISIBLE 0)
  (ani-time t)
  (ani-properties ANI_VISIBLE 1)
  (while (and (<= x bounce-width) (>= x 0) (<= t bounce-duration))
    (ani-time t)
    (ani-properties ANI_X x
		    ANI_Y (- bounce-height (+ y (* yd (sin (* 10 x))))))
    (set! x (+ x xd))
    (set! t (+ t td)))
  (ani-properties ANI_VISIBLE 0))

(define (bounce)
  (ani-geometry bounce-width bounce-height)
  (ani-duration bounce-duration)
  (ani-background "window.xpm")
  (ani-object ANI_FILLELLIPSE)
  (ani-properties ANI_COLOR WHITE)
  (draw-bounce 0 0 20 100 100 0 200)
  (ani-object ANI_FILLRECT)
  (ani-properties ANI_COLOR YELLOW)
  (draw-bounce 10 0 30 150 150 200 200)
  (ani-object ANI_ELLIPSE)
  (ani-properties ANI_COLOR GREEN)
  (draw-bounce 590 0 -40 170 170 400 200)
  (ani-object ANI_STRING)
  (ani-properties ANI_TEXT "Egon Animator"
		  ANI_FONT (+ TIMES SIZE_30 BOLD)
		  ANI_COLOR MAGENTA)
  (draw-sin 590 300 -25 80 600 200)
  (ani-object ANI_STRING)
  (ani-properties ANI_TEXT "(C) 1997 Ulric Eriksson"
		  ANI_FONT (+ COURIER SIZE_30 BOLD)
		  ANI_COLOR RED)
  (draw-sin 0 200 25 80 800 200)
  (ani-object ANI_STRING)
  (ani-properties ANI_TEXT "No Warranty"
		  ANI_FONT (+ HELVETICA SIZE_30 BOLD)
		  ANI_COLOR BLUE)
  (draw-sin 600 100 -25 80 1000 200))

(bounce)

